/****************************************************************************
 ****************************************************************************/

#include "jwm.h"

unsigned long *colors = NULL;

static int ParseColor(int index);
static int ReadHex(const char *hex);
static void InitializeNames();

static char **names = NULL;

/****************************************************************************
 ****************************************************************************/
void InitializeColors() {
	XColor ideal, temp;
	Colormap cm;
	int x;

	cm = DefaultColormap(display, 0);
	colors = Allocate(sizeof(unsigned long) * COLOR_COUNT);

	if(!ParseColor(COLOR_BORDER)) {
		XAllocNamedColor(display, cm, "gray", &temp, &ideal);
		colors[COLOR_BORDER] = temp.pixel;
	}

	if(!ParseColor(COLOR_BORDER_OUTLINE)) {
		XAllocNamedColor(display, cm, "black", &temp, &ideal);
		colors[COLOR_BORDER_OUTLINE] = temp.pixel;
	}

	if(!ParseColor(COLOR_BORDER_TEXT)) {
		XAllocNamedColor(display, cm, "black", &temp, &ideal);
		colors[COLOR_BORDER_TEXT] = temp.pixel;
	}

	if(!ParseColor(COLOR_ACTIVE_BORDER)) {
		XAllocNamedColor(display, cm, "blue", &temp, &ideal);
		colors[COLOR_ACTIVE_BORDER] = temp.pixel;
	}

	if(!ParseColor(COLOR_ACTIVE_BORDER_TEXT)) {
		XAllocNamedColor(display, cm, "white", &temp, &ideal);
		colors[COLOR_ACTIVE_BORDER_TEXT] = temp.pixel;
	}

	if(!ParseColor(COLOR_MOUSE_FOREGROUND)) {
		XAllocNamedColor(display, cm, "black", &temp, &ideal);
		colors[COLOR_MOUSE_FOREGROUND] = temp.pixel;
	}

	if(!ParseColor(COLOR_MOUSE_BACKGROUND)) {
		XAllocNamedColor(display, cm, "white", &temp, &ideal);
		colors[COLOR_MOUSE_BACKGROUND] = temp.pixel;
	}

	if(names) {
		for(x = 0; x < COLOR_COUNT; x++) {
			if(names[x]) {
				Release(names[x]);
			}
		}
		Release(names);
		names = NULL;
	}

}

/****************************************************************************
 ****************************************************************************/
void ReleaseColors() {
	int x;

	if(colors) {
		XFreeColors(display, DefaultColormap(display, 0), colors,
			COLOR_COUNT, 0);
		Release(colors);
	}
	if(names) {
		for(x = 0; x < COLOR_COUNT; x++) {
			if(names[x]) {
				Release(names[x]);
			}
		}
		Release(names);
		names = NULL;
	}
}

/****************************************************************************
 ****************************************************************************/
void SetTextColor(const char *value) {
	InitializeNames();
	names[COLOR_BORDER_TEXT] = Allocate(strlen(value) + 1);
	strcpy(names[COLOR_BORDER_TEXT], value);
}

/****************************************************************************
 ****************************************************************************/
void SetActiveTextColor(const char *value) {
	InitializeNames();
	names[COLOR_ACTIVE_BORDER_TEXT] = Allocate(strlen(value) + 1);
	strcpy(names[COLOR_ACTIVE_BORDER_TEXT], value);
}

/****************************************************************************
 ****************************************************************************/
void SetBorderColor(const char *value) {
	InitializeNames();
	names[COLOR_BORDER] = Allocate(strlen(value) + 1);
	strcpy(names[COLOR_BORDER], value);
}

/****************************************************************************
 ****************************************************************************/
void SetActiveBorderColor(const char *value) {
	InitializeNames();
	names[COLOR_ACTIVE_BORDER] = Allocate(strlen(value) + 1);
	strcpy(names[COLOR_ACTIVE_BORDER], value);
}

/****************************************************************************
 ****************************************************************************/
int ParseColor(int index) {
	XColor temp, ideal;
	Colormap cm;
	int rgb;

	if(!names || !names[index]) {
		return 0;
	}

	cm = DefaultColormap(display, 0);

	if(names[index][0] == '#' && strlen(names[index]) == 7) {
		rgb = ReadHex(names[index] + 1);
		temp.red = ((rgb >> 16) & 0xFF) * 257;
		temp.green = ((rgb >> 8) & 0xFF) * 257;
		temp.blue = (rgb & 0xFF) * 257;
		temp.flags = DoRed | DoGreen | DoBlue;
		XAllocColor(display, cm, &temp);
	} else {
		if(!XAllocNamedColor(display, cm, names[index], &temp, &ideal)) {
			Warning("bad color: \"%s\"", names[index]);
			return 0;
		}
	}
	colors[index] = temp.pixel;
	return 1;
}

/****************************************************************************
 ****************************************************************************/
void InitializeNames() {
	int x;

	if(names == NULL) {
		names = Allocate(sizeof(char*) * COLOR_COUNT);
		for(x = 0; x < COLOR_COUNT; x++) {
			names[x] = NULL;
		}
	}
}

/****************************************************************************
 ****************************************************************************/
int ReadHex(const char *hex) {
	int value = 0;
	int x;

	for(x = 0; hex[x]; x++) {
		value *= 16;
		if(hex[x] >= '0' && hex[x] <= '9') {
			value += hex[x] - '0';
		} else if(hex[x] >= 'A' && hex[x] <= 'F') {
			value += hex[x] - 'A' + 10;
		} else if(hex[x] >= 'a' && hex[x] <= 'f') {
			value += hex[x] - 'a' + 10;
		}
	}

	return value;
}



